<?php
/*
	---------------------------
	All the admin functionality
	---------------------------
*/



class sllimboAdmin {



	public function __construct() {

		add_action( 'admin_init',                  array( $this, 'redirect'              ) );
		add_action( 'admin_head',                  array( $this, 'css'                   ) );
		add_action( 'admin_menu',                  array( $this, 'admin_menu'            ), 1000 );
		add_action( 'admin_menu',                  array( $this, 'remove_meta_boxes'     ) );
		add_action( 'admin_init',                  array( $this, 'register_admin_styles' ) );
		add_action( 'admin_print_styles',          array( $this, 'add_admin_styles'      ) );
		add_action( 'admin_enqueue_scripts',       array( $this, 'add_admin_scripts'     ) );
		add_filter( 'tiny_mce_before_init',        array( $this, 'wysiwyg_defaults'      ) );
		add_action( 'add_meta_boxes',              array( $this, 'add_meta_boxes'        ) );
		add_action( 'save_post',                   array( $this, 'save_meta'             ), 10, 2 );

		add_filter( 'admin_footer_text',           array( $this, 'admin_footer'          ) );
		add_filter( 'manage_users_columns',        array( $this, 'user_columns'          ) );
		add_filter( 'manage_users_custom_column',  array( $this, 'user_column'           ), 10, 3 );

	}



	function redirect() {

		if ( ! current_user_can( 'edit_posts' ) ) {
			wp_redirect( site_url() );
			exit;
		}

	}



	function css() {

		echo '<style>.sllimbo-shortcode-highlight { background: #f00 !important; }</style>';

	}



	function register_admin_styles() {

		wp_register_style(
			'sllimboAdmin',
			SLLIMBO_PLUGIN_DIR . 'css/admin.css',
			null,
			filemtime( SLLIMBO_PLUGIN_PATH . 'css/admin.css' )
			);

		wp_register_style(
			'sllimboAdminDatePick',
			SLLIMBO_PLUGIN_DIR . 'css/admin-datepick.css',
			null,
			filemtime( SLLIMBO_PLUGIN_PATH . 'css/admin-datepick.css' )
			);

	}



	function add_admin_styles() {

		wp_enqueue_style( 'sllimboAdminDatePick' );

		if ( ! current_user_can( 'use_wordpress_admin' ) ) {
			wp_enqueue_style( 'sllimboAdmin' );
		}

	}



	function add_admin_scripts() {

		wp_enqueue_script(
			'sllimboAdminDatePick',
			SLLIMBO_PLUGIN_DIR . 'js/jquery.datepick.min.js',
			array( 'jquery' ),
			filemtime( SLLIMBO_PLUGIN_PATH . 'js/jquery.datepick.min.js' )
			);

	}



	function admin_menu() {

		global $menu, $submenu;

		if ( ! current_user_can( 'use_wordpress_admin' ) ) {

			remove_menu_page( 'edit-comments.php' );
			remove_menu_page( 'upload.php' );
			remove_menu_page( 'tools.php' );
			remove_menu_page( 'edit.php?post_type=soliloquy' );
			remove_menu_page( 'edit.php?post_type=local-email' );

		}

	}



	function remove_meta_boxes() {

		// dump dashboard crap
		remove_meta_box( 'dashboard_recent_comments', 'dashboard', 'core' );
		remove_meta_box( 'dashboard_incoming_links',  'dashboard', 'core' );
		remove_meta_box( 'dashboard_recent_drafts',   'dashboard', 'core' );
		remove_meta_box( 'dashboard_quick_press',     'dashboard', 'core' );
		remove_meta_box( 'dashboard_plugins',         'dashboard', 'core' );
		remove_meta_box( 'dashboard_primary',         'dashboard', 'core' );
		remove_meta_box( 'dashboard_secondary',       'dashboard', 'core' );

		// clean up post edit screen
		remove_meta_box( 'postexcerpt', 'post', 'core' );
		remove_meta_box( 'formatdiv', 'post', 'core' );
		remove_meta_box( 'categorydiv', 'post', 'core' );
		// remove_meta_box( 'tagsdiv-post_tag',   'post', 'core' );
		remove_meta_box( 'commentsdiv', 'post', 'core' );
		remove_meta_box( 'postcustom', 'post', 'core' );
		remove_meta_box( 'trackbacksdiv', 'post', 'core' );
		remove_meta_box( 'slugdiv', 'post', 'core' );
		remove_meta_box( 'authordiv', 'post', 'core' );

		// clean up page edit screen
		remove_meta_box( 'pageparentdiv', 'page', 'core'  );
		remove_meta_box( 'commentsdiv', 'page', 'core'  );
		remove_meta_box( 'postcustom', 'page', 'core' );

	}



	function add_meta_boxes() {

		global $post;

		if ( $post->ID != get_option( 'page_on_front' ) ) {
			add_meta_box(
				'sllimbo-parent',
				'Section',
				array( $this, 'parent_meta' ),
				'page',
				'side'
				);
		}

		add_meta_box(
			'sllimbo-categories',
			'Category',
			array( $this, 'category_meta' ),
			'post',
			'side'
			);

		add_meta_box(
			'sllimbo-phrases',
			'Comments',
			array( $this, 'phrase_meta' ),
			'post',
			'normal'
			);

		add_meta_box(
			'sllimbo-page',
			'Visibility',
			array( $this, 'page_meta' ),
			'page',
			'normal'
			);

		add_meta_box(
			'sllimbo-details',
			'Details',
			array( $this, 'article_meta' ),
			'post',
			'normal'
			);

	}



	function parent_meta() {

		global $post;

		$pages = wp_dropdown_pages( array(
										'post_type'        => 'page',
										'exclude_tree'     => $post->ID,
										'selected'         => $post->post_parent,
										'name'             => 'parent_id',
										'depth'            => 1,
										'show_option_none' => 'Top Level (no parent)',
										'sort_column'      => 'menu_order, post_title',
										'echo'             => 0
										) );
		?>

		<p><?php echo $pages; ?></p>

		<?php

	}



	function category_meta() {

		global $post;

		$selected = wp_get_object_terms( $post->ID, 'category', array( 'fields' => 'ids' ) );
		$walker   = new sllimboWalker;
		$cats = wp_terms_checklist(
						$post->ID,
						array(
							'taxonomy'      => 'category',
							'walker'        => $walker,
							'selected_cats' => $selected,
							'checked_ontop' => true,
							'echo'          => false
							)
						);

		?>

		<?php echo $cats; ?>

		<?php

	}



	function phrase_meta( $post ) {

		$phrases = sllimbo::custom( 'phrases' );

		?>

		<div class="sllimbo-meta" style="font-size:1.4em;line-height:1.6;padding:10px;max-width:640px;">

			<p><em>Any changes you have made to comments will only be shown here after you save the post.</em></p>

			<?php if ( ! $phrases ) {
				echo '<p><em>No comments currently defined.</em></p>';
			} else {
				foreach ( $phrases as $phrase ) {
					echo '<p>';
					printf( '<span style="background:#46abbd;color:#fff;padding:3px;text-transform:uppercase;font-weight:bold;">%s</span>', ( $phrase['alt'] ? $phrase['alt'] : $phrase['term'] ) );
					printf( '<br>%s', $phrase['def'] );
					if ( isset( $phrase['synonym'] ) ) {
						printf( '<br><em>Also:</em> %s', $phrase['synonym'] );
					}
					echo '</p>';
				}
			} ?>
		</div>

		<?php

	}



	function article_meta( $post ) {

		$source = sllimbo::custom( 'source' );
		$url    = sllimbo::custom( 'url' );
		$date   = sllimbo::custom( 'date' );
		$num    = sllimbo::custom( 'num' );
		$free   = sllimbo::custom( 'free' );

		if ( $date ) {
			$date = mysql2date( 'M d, Y', $date );
		}

		?>

		<div class="sllimbo-meta">

			<p><label for="sllimbo-source">Source</label>
			<input type="text" id="sllimbo-source" name="sllimbo-source" value="<?php echo $source; ?>"></p>

			<p><label for="sllimbo-url">Original URL</label>
			<input type="text" id="sllimbo-url" name="sllimbo-url" value="<?php echo $url; ?>"></p>

			<p><label for="sllimbo-date">Original article date</label>
			<input type="text" id="sllimbo-date" name="sllimbo-date" value="<?php echo $date; ?>"></p>
			<script>
				jQuery('#sllimbo-date').datepick({
					alignment:  'top',
					dateFormat: 'M d, yyyy',
					maxDate: 0
				});
			</script>

			<p><label for="sllimbo-num">Number</label>
			<select id="sllimbo-num" name="sllimbo-num">
				<?php for ( $i = 1; $i < 21; $i++ ) { ?>
					<option value="<?php echo $i; ?>" <?php selected( $i, $num ); ?>><?php echo $i; ?></option>
				<?php } ?>
			</select>
			</p>

			<p><label for="sllimbo-free">Free?</label>
			<input type="checkbox" id="sllimbo-free" name="sllimbo-free" value="1" <?php checked( $free ); ?>></p>

			<?php wp_nonce_field( 'sllimbo_article_meta', 'sllimbo-article' ); ?>

		</div>

		<?php

	}



	function page_meta( $post ) {

		$hidden = sllimbo::custom( 'logged-in-only' );

		?>

		<div class="sllimbo-meta">

			<p><label for="sllimbo-free">Logged-in only</label>
			<input type="checkbox" id="sllimbo-logged-in-only" name="sllimbo-logged-in-only" value="1" <?php checked( $hidden ); ?>></p>

			<?php wp_nonce_field( 'sllimbo_page_meta', 'sllimbo-page' ); ?>

		</div>

		<?php

	}



	function save_meta( $post_id, $post ) {

		if (
			!current_user_can( 'edit_post', $post_id )
			|| ( defined( 'DOING_AJAX' ) and DOING_AJAX )
			|| wp_is_post_revision( $post_id )
			|| wp_is_post_autosave( $post_id )
			|| 'auto-draft' == $post->post_status
			|| 'trash' == $post->post_status
			) {
			return $post_id;
		}

		if ( 'post' == $post->post_type ) {

			$pattern = get_shortcode_regex();
			$phrases = array();

			preg_match_all( "/$pattern/s", $post->post_content, $matches );

			if ( $matches ) {

				foreach ( $matches[2] as $k => $v ) {
					if ( 'phrase' == $v ) {
						$phrases[] = shortcode_parse_atts( $matches[3][$k] );
					}
				}

			}

			delete_post_meta( $post_id, 'sllimbo-phrases' );
			delete_post_meta( $post_id, 'sllimbo-phrase' );

			if ( ! empty( $phrases ) ) {
				update_post_meta( $post_id, 'sllimbo-phrases', $phrases );
				foreach ( $phrases as $phrase ) {
					add_post_meta( $post_id, 'sllimbo-phrase', $phrase['term'] );
				}
			}

		}

		$meta = array();

		if ( wp_verify_nonce( $_POST['sllimbo-article'], 'sllimbo_article_meta' ) ) {

			$meta[] = 'source';
			$meta[] = 'url';
			$meta[] = 'date';
			$meta[] = 'num';
			$meta[] = 'free';

		}

		if ( wp_verify_nonce( $_POST['sllimbo-page'], 'sllimbo_page_meta' ) ) {

			$meta[] = 'logged-in-only';

		}

		if ( ! empty( $meta ) ) {
			foreach ( $meta as $field ) {
				$field = 'sllimbo-' . $field;
				delete_post_meta( $post_id, $field );
				if ( isset( $_POST[$field] ) and '' != trim( $_POST[$field] ) ) {
					$data = trim( $_POST[$field] );
					$data = apply_filters( 'wp_filter_post_kses', $data );
					if ( 'sllimbo-url' == $field ) {
						if ( 0 !== strpos( $data, 'http://' ) and 0 !== strpos( $data, 'https://' ) ) {
							$data = 'http://' . $data;
						}
					}
					if ( 'sllimbo-date' == $field ) {
						$data = date( 'Y-m-d 00:00:00', strtotime( $data ) );
					}
					add_post_meta( $post_id, $field, $data );
				}
			}
		}

	}



	function do_shortcode_tag( $m ) {

		if ( 'phrase' != $m[2] )
			return;

		$atts = shortcode_parse_atts( $m[3] );

		return $atts;

	}



	function wysiwyg_defaults( $initArray ) {

		// see http://wiki.moxiecode.com/index.php/TinyMCE:Control_reference

		if ( ! current_user_can( 'use_wordpress_admin' ) ) {

			$initArray['theme_advanced_blockformats'] = 'p,h2,h3';

			$initArray['theme_advanced_buttons1'] = 'pastetext,|,|,formatselect,|,|,sllimbophrase,|,|,bold,italic,|,|,bullist,numlist,|,|,link,unlink,|,|,spellchecker,|,|,fullscreen';
			$initArray['theme_advanced_buttons2'] = '';
			$initArray['theme_advanced_buttons3'] = '';

			$initArray['spellchecker_languages'] = 'English=en';

		}

		return $initArray;

	}



	function admin_footer( $default_text ) {

		return '<span id="footer-thankyou">Developed by <a href="http://lumpylemon.co.uk">Lumpy Lemon</a><span> | Powered by <a href="http://wordpress.org">WordPress</a>';

	}



	function user_columns( $cols ) {

		unset( $cols['posts'] );

		$cols['user-joined']  = 'Joined';
		$cols['user-country'] = 'Country';

		return $cols;

	}



	function user_column( $empty = '', $col, $user_id ) {

		switch ( $col ) {

			case 'user-joined' :
				if ( ! user_can( $user_id, 'edit_posts' ) ) {
					$userdata = get_userdata( (int) $user_id );
					return mysql2date( get_option( 'date_format' ), $userdata->user_registered );
				}
			break;

			case 'user-country' :
				if ( ! user_can( $user_id, 'edit_posts' ) ) {
					return get_user_meta( $user_id, 'sllimbo-country', true );
				}
			break;

		}

	}



}



class sllimboWalker extends Walker {



	var $tree_type = 'category';
	var $db_fields = array(
						'parent' => 'parent',
						'id'     => 'term_id'
						);



	function start_lvl( &$output, $depth, $args ) {

		$output .= "<p>";

	}



	function end_lvl( &$output, $depth, $args ) {

		$output .= "</p>\n";

	}



	function start_el( &$output, $term, $depth, $args ) {

		$name = 'tax_input[' . $args['taxonomy'] . ']';
		$output .= '<label class="selectit">' .
			'<input value="' . $term->term_id . '" type="radio" name="'.$name.'[]" ' .
				'id="in-'.$args['taxonomy'].'-' . $term->term_id . '"' .
				checked( in_array( $term->term_id, $args['selected_cats'] ), true, false ) .
				disabled( empty( $args['disabled'] ), false, false ) .
			'> ' .
			esc_html( apply_filters( 'the_category', $term->name ) ) .
			'</label>';

	}



	function end_el( &$output, $term, $depth, $args ) {
		$output .= "<br>\n";
	}



}



$sllimbo_admin = new sllimboAdmin;



?>