<?php
/*
Plugin Name: sllimbo
Plugin URI:  http://sllimbo.com
Description: Core functions for the sllimbo website
Version:     0.1
Author:      Simon Blackbourn @ Lumpy Lemon
Author URI:  http://lumpylemon.co.uk

	This plugin consists of a number of included files
	to provide all the core functionality
	for the sllimbo website
	http://sllimbo.com

	All code by Simon Blackbourn @ by Lumpy Lemon (unless credited otherwise)
	http://lumpylemon.co.uk
	Twitter + Skype: lumpysimon

	Copyright (c) Lumpy Lemon Ltd. All rights reserved.

	Released under the GPL license:
	http://www.opensource.org/licenses/gpl-license.php

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

*/



if ( ! defined( 'SLLIMBO_PLUGIN_PATH' ) )
	define( 'SLLIMBO_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );

if ( ! defined( 'SLLIMBO_PLUGIN_DIR' ) )
	define( 'SLLIMBO_PLUGIN_DIR', plugin_dir_url( __FILE__ ) );



foreach ( glob( dirname( __FILE__ ) . '/inc/*.php' ) as $file ) {
	require $file;
}



class sllimboCore {



	public function __construct() {

		register_activation_hook(   __FILE__, array( $this, 'hello'   ) );
		register_deactivation_hook( __FILE__, array( $this, 'goodbye' ) );

		add_action( 'init',                       array( $this, 'post_types' ) );
		add_action( 'init',                       array( $this, 'taxonomies' ) );

		// add_filter( 'bwp_gxs_post_where',          array( $this, 'sitemap' ), 10, 2 );

	}



	function hello() {

		if ( ! $member = get_role( 'current_member' ) ) {
			$member = add_role( 'current_member', 'Current Member', array( 'read' => true ) );
		}

		if ( ! $inactive_member = get_role( 'inactive_member' ) ) {
			$inactive_member = add_role( 'inactive_member', 'Inactive Member', array( 'read' => true ) );
		}

		$allowed = array( 'administrator', 'author', 'contributor', 'editor', 'current_member' );

		foreach ( $allowed as $allow ) {
			if ( $role = get_role( $allow ) ) {
				$role->add_cap( 'access_articles' );
			}
		}

		// enable admin to see all admin screen functionality
		if ( $admin = get_role( 'administrator' ) ) {
			$admin->add_cap( 'use_wordpress_admin' );
		}

		// remove ability for non-admins to manage links
		if ( $editor = get_role( 'editor' ) ) {
			$editor->remove_cap( 'manage_links' );
			$editor->remove_cap( 'moderate_comments' );
		}

	}



	function goodbye() {

		// remove ability for admin to see all admin screen functionality
		$admin = get_role( 'administrator' );
		if ( ! empty( $admin ) ) {
			$admin->remove_cap( 'use_wordpress_admin' );
		}

		// if there are no current members, remove that role
		$users = get_users( array( 'role' => 'current_member' ) );
		if ( count( $users ) <= 0 ) {
			remove_role( 'current_member' );
		}

		// if there are no inactive members, remove that role
		$users = get_users( array( 'role' => 'inactive_member' ) );
		if ( count( $users ) <= 0 ) {
			remove_role( 'inactive_member' );
		}

		// if there are no ex members, remove that role
		$users = get_users( array( 'role' => 'ex_member' ) );
		if ( count( $users ) <= 0 ) {
			remove_role( 'ex_member' );
		}

	}



	function post_types() {

		remove_post_type_support( 'post', 'comments' );
		remove_post_type_support( 'post', 'thumbnail' );

		if ( function_exists( 'register_extended_post_type' ) ) {

			$quick_edit = current_user_can( 'activate_plugins' );

			register_extended_post_type(
				'post',
				array(
					'cols' => array(
						'pub-date' => array(
							'title'   => 'Published',
							'default' => 'desc',
							'post_field' => 'post_date'
						),
						'mod-date' => array(
							'title'   => 'Modified',
							'default' => 'desc',
							'post_field' => 'post_modified'
						),
						'article-date' => array(
							'title'    => 'Original article',
							'date_format' => true,
							'meta_key' => 'sllimbo-date'
						),
						'number' => array(
							'title'    => 'No.',
							'sortable' => false,
							'meta_key' => 'sllimbo-num'
						),
						'source' => array(
							'title'    => 'Source',
							'sortable' => false,
							'meta_key' => 'sllimbo-source'
						),
						'phrases' => array(
							'title'    => 'Comments',
							'function' => function() {
								if ( $phrases = sllimbo::custom( 'phrases' ) )
									echo count( $phrases );
							}
						),
						'cat'  => array(
							'title' => 'Category',
							'taxonomy' => 'category'
							),
						'free' => array(
							'title'    => 'Free',
							'meta_key' => 'sllimbo-free',
							'sortable' => false,
							'function' => function() {
								if ( sllimbo::custom( 'free' ) )
									echo '&#10003;';
							}
						)
					),
					'filters' => array(
						'post_is' => array(
							'meta_exists' => array(
								'sllimbo-free'  => 'Free'
							)
						)
					),
					'right_now' => true,
					'quick_edit' => $quick_edit
					)
				);

			register_extended_post_type(
				'page',
				array(
					'cols' => array(
						'logged-in-only' => array(
							'title'    => 'Logged-in only',
							'meta_key' => 'sllimbo-logged-in-only',
							'sortable' => false,
							'function' => function() {
								if ( sllimbo::custom( 'logged-in-only' ) )
									echo '&#10003;';
							}
						)
					),
					'right_now' => true,
					'quick_edit' => $quick_edit
					)
				);

			register_extended_post_type(
				'payment',
				array(
					'capability_type' => 'post',
					'hierarchical'    => false,
					'menu_position'   => 20,
					'right_now'       => true,
					'public'          => true,
					'publicly_queryable' => false,
					'quick_edit'      => $quick_edit,
					'show_ui'         => true,
					'supports'        => array( 'title', 'editor' ),
					'cols' => array(
						'date' => array(
							'title' => 'Date'
							),
						'payment-amount' => array(
							'title' => 'Amount',
							'meta_key' => 'sllimbo-payment-amount',
							'sortable' => false,
							function() {
								if ( $amount = sllimbo::custom( 'payment-amount' ) ) {
									return '&pound;' . $amount;
								}
							}
							),
						'payment-registrant-name' => array(
							'title' => 'Name',
							'meta_key' => 'sllimbo-payment-subscriber-name'
							),
						'payment-registrant-id' => array(
							'title' => 'Member ID',
							'meta_key' => 'sllimbo-payment-subscriber-id'
							),
						'payment-id' => array(
							'title' => 'ID',
							'meta_key' => 'sllimbo-payment-transaction-id'
									),
						'payment-type' => array(
							'title' => 'Type',
							'taxonomy' => 'transaction-type'
							)
						)
					)
				);

			register_extended_post_type(
				'sidebox',
				array(
					'capability_type'    => 'post',
					'hierarchical'       => false,
					'menu_position'      => 20,
					'right_now'          => true,
					'public'             => false,
					'publicly_queryable' => false,
					'quick_edit'         => $quick_edit,
					'show_ui'            => true,
					'supports'           => array( 'title', 'editor' ),
					'cols' => array(
						'sidebox-content' => array(
							'title'      => 'Text',
							'post_field' => 'post_content',
							'sortable'   => false
							)
						)
					),
				'sideboxes',
				'sidebox'
				);

		}

	}



	function taxonomies() {

		if ( function_exists( 'register_extended_taxonomy' ) ) {

			register_extended_taxonomy(
								'method',
								'payment',
								array(
									'hierarchical' => false
									)
								);

			register_extended_taxonomy(
								'transaction-type',
								'payment',
								array(
									'hierarchical' => false
									)
								);

		}

	}



	// exclude certain items from the sitemap
	function sitemap( $query, $post_type ) {

		$exclude_parents = array(
								get_option( 'page_on_front' ),
								6316,
								6318
								);

		$exclude_ids     = array(
								1422,
								6032,
								6316,
								6318
								);

		$exclude_parents_str = implode( ',', $exclude_parents );
		$exclude_ids_str     = implode( ',', $exclude_ids );

		switch ( $post_type ) {

			case 'page' :
				return ' AND wposts.ID NOT IN (' . $exclude_ids_str . ') AND wposts.post_parent NOT IN (' . $exclude_parents_str . ') ';
			break;

		}

		return '';

	}



}



$sllimbo_core = new sllimboCore;



?>